/*
 * Copyright (c) 2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "io.h"
#include "serial.h"
#include "atmel_usart.h"

static struct serial_port *active_port;

void init_serial(void)
{
	struct serial_port *port;
	unsigned int i;

	/*
	 * Look through all serial ports to see if one of them may
	 * have been initialized by u-boot. Pick the first one.
	 */
	for (i = 0; i < nr_serial_ports; i++) {
		unsigned long brgr;

		port = &serial[i];
		brgr = usart_readl(port, BRGR);
		if (USART_BFEXT(CD, brgr)) {
			active_port = port;
			break;
		}
	}

	if (!active_port) {
		/* Ok, didn't find any. Bail out. */
		for (;;);
	}
}

int serial_putchar(int c)
{
	unsigned char ch = c;

	if (ch == '\n')
		serial_putchar('\r');

	while (!(usart_readl(active_port, CSR) & USART_BIT(TXRDY)))
		;
	usart_writel(active_port, THR, ch);
	return ch;
}

int serial_puts(const char *s)
{
	while (*s)
		serial_putchar(*s++);
	return 0;
}

int serial_poll(void)
{
	return !!(usart_readl(active_port, CSR) & USART_BIT(RXRDY));
}

int serial_getchar(void)
{
	while (!(usart_readl(active_port, CSR) & USART_BIT(RXRDY)))
		;
	return usart_readl(active_port, RHR) & 0xff;
}
