/*
 * Copyright (c) 2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __UTIL_H__
#define __UTIL_H__

#include <types.h>

/*
 * Compiler-dependent stuff. We don't support anything but gcc for now.
 */
#define __noreturn __attribute__((noreturn))

#define ARRAY_SIZE(__a)		(sizeof(__a) / sizeof((__a)[0]))

extern unsigned char _text[];
extern unsigned char _etext[];
extern unsigned char _data[];
extern unsigned char _edata[];
extern unsigned char _end[];

extern unsigned char image_start[];
extern unsigned char image_end[];

void *memset(void *p, int c, size_t n);
void *memcpy(void *dst, void *src, size_t n);

/*
 * Note that malloc() will panic if the request can't be satisfied, so
 * it's safe to not check for NULL.
 */
void *malloc(unsigned int size);
void *zalloc(unsigned int size);

void print(const char *s, int ch);
void print_dec(unsigned long n, int ch);
void print_hex(unsigned long n, unsigned int width, int ch);
void __noreturn panic(const char *msg);

void udelay(unsigned long us);

/*
 * Byte swapping
 */
static inline u16 le16_to_cpu(u16 x)
{
	return __builtin_bswap_16(x);
}

static inline u32 le32_to_cpu(u32 x)
{
	return __builtin_bswap_32(x);
}

static inline u16 cpu_to_le16(u16 x)
{
	return __builtin_bswap_16(x);
}

static inline u32 cpu_to_le32(u32 x)
{
	return __builtin_bswap_32(x);
}

static inline u16 be16_to_cpu(u16 x)
{
	return x;
}

static inline u32 be32_to_cpu(u32 x)
{
	return x;
}

static inline u16 cpu_to_be16(u16 x)
{
	return x;
}

static inline u32 cpu_to_be32(u32 x)
{
	return x;
}

#endif /* __UTIL_H__ */
